/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https: //mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.client

import androidx.room.Room
import androidx.sqlite.driver.bundled.BundledSQLiteDriver
import dagger.Binds
import dagger.Component
import dagger.Module
import de.justjanne.libquassel.backend.AliasManagerPersister
import de.justjanne.libquassel.backend.BacklogManagerPersister
import de.justjanne.libquassel.backend.BufferSyncerPersister
import de.justjanne.libquassel.backend.BufferViewConfigPersister
import de.justjanne.libquassel.backend.BufferViewManagerPersister
import de.justjanne.libquassel.backend.CertManagerPersister
import de.justjanne.libquassel.backend.CoreInfoPersister
import de.justjanne.libquassel.backend.HighlightRuleManagerPersister
import de.justjanne.libquassel.backend.IdentityPersister
import de.justjanne.libquassel.backend.IgnoreListManagerPersister
import de.justjanne.libquassel.backend.IrcChannelPersister
import de.justjanne.libquassel.backend.IrcListHelperPersister
import de.justjanne.libquassel.backend.IrcUserPersister
import de.justjanne.libquassel.backend.NetworkConfigPersister
import de.justjanne.libquassel.backend.NetworkPersister
import de.justjanne.libquassel.backend.RpcPersister
import de.justjanne.libquassel.persistence.AppDatabase
import de.justjanne.libquassel.persistence.TodoEntity
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.api.client.AliasManagerClientApi
import de.justjanne.libquassel.protocol.api.client.BacklogManagerClientApi
import de.justjanne.libquassel.protocol.api.client.BufferSyncerClientApi
import de.justjanne.libquassel.protocol.api.client.BufferViewConfigClientApi
import de.justjanne.libquassel.protocol.api.client.BufferViewManagerClientApi
import de.justjanne.libquassel.protocol.api.client.CertManagerClientApi
import de.justjanne.libquassel.protocol.api.client.CoreInfoClientApi
import de.justjanne.libquassel.protocol.api.client.HighlightRuleManagerClientApi
import de.justjanne.libquassel.protocol.api.client.IdentityClientApi
import de.justjanne.libquassel.protocol.api.client.IgnoreListManagerClientApi
import de.justjanne.libquassel.protocol.api.client.IrcChannelClientApi
import de.justjanne.libquassel.protocol.api.client.IrcListHelperClientApi
import de.justjanne.libquassel.protocol.api.client.IrcUserClientApi
import de.justjanne.libquassel.protocol.api.client.NetworkClientApi
import de.justjanne.libquassel.protocol.api.client.NetworkConfigClientApi
import de.justjanne.libquassel.protocol.api.client.RpcClientApi
import de.justjanne.libquassel.protocol.api.dispatcher.ClientDispatcherModule
import de.justjanne.libquassel.protocol.api.dispatcher.RpcDispatcher
import de.justjanne.libquassel.protocol.api.dispatcher.SyncHandler
import de.justjanne.libquassel.protocol.api.proxy.ClientProxyModule
import de.justjanne.libquassel.protocol.api.proxy.Proxy
import de.justjanne.libquassel.protocol.api.server.AliasManagerServerApi
import de.justjanne.libquassel.protocol.api.server.BacklogManagerServerApi
import de.justjanne.libquassel.protocol.api.server.BufferSyncerServerApi
import de.justjanne.libquassel.protocol.api.server.BufferViewConfigServerApi
import de.justjanne.libquassel.protocol.api.server.BufferViewManagerServerApi
import de.justjanne.libquassel.protocol.api.server.CertManagerServerApi
import de.justjanne.libquassel.protocol.api.server.HighlightRuleManagerServerApi
import de.justjanne.libquassel.protocol.api.server.IdentityServerApi
import de.justjanne.libquassel.protocol.api.server.IgnoreListManagerServerApi
import de.justjanne.libquassel.protocol.api.server.IrcListHelperServerApi
import de.justjanne.libquassel.protocol.api.server.NetworkConfigServerApi
import de.justjanne.libquassel.protocol.api.server.NetworkServerApi
import de.justjanne.libquassel.protocol.exceptions.RpcInvocationFailedException
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.MsgId
import de.justjanne.libquassel.protocol.models.types.QtType
import de.justjanne.libquassel.protocol.variant.QVariantList
import de.justjanne.libquassel.protocol.variant.QVariant_
import de.justjanne.libquassel.protocol.variant.qVariant
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.runBlocking
import org.junit.jupiter.api.Test
import javax.inject.Inject
import javax.inject.Singleton
import kotlin.test.assertFails
import kotlin.test.assertFailsWith

@Singleton
class ProxyImpl @Inject constructor() : Proxy {
  override fun sync(className: String, objectName: ObjectName, function: String, params: QVariantList) {
    println("making sync call: $className $objectName $function $params")
  }

  override fun rpc(function: String, params: QVariantList) {
    println("making rpc call: $function $params")
  }
}

@Module
interface ClientModule {
  @Binds fun bindAliasManagerPersister(impl: AliasManagerPersister): AliasManagerClientApi
  @Binds fun bindBacklogManagerPersister(impl: BacklogManagerPersister): BacklogManagerClientApi
  @Binds fun bindBufferSyncerPersister(impl: BufferSyncerPersister): BufferSyncerClientApi
  @Binds fun bindBufferViewConfigPersister(impl: BufferViewConfigPersister): BufferViewConfigClientApi
  @Binds fun bindBufferViewManagerPersister(impl: BufferViewManagerPersister): BufferViewManagerClientApi
  @Binds fun bindCertManagerPersister(impl: CertManagerPersister): CertManagerClientApi
  @Binds fun bindCoreInfoPersister(impl: CoreInfoPersister): CoreInfoClientApi
  @Binds fun bindHighlightRuleManagerPersister(impl: HighlightRuleManagerPersister): HighlightRuleManagerClientApi
  @Binds fun bindIdentityPersister(impl: IdentityPersister): IdentityClientApi
  @Binds fun bindIgnoreListManagerPersister(impl: IgnoreListManagerPersister): IgnoreListManagerClientApi
  @Binds fun bindIrcChannelPersister(impl: IrcChannelPersister): IrcChannelClientApi
  @Binds fun bindIrcListHelperPersister(impl: IrcListHelperPersister): IrcListHelperClientApi
  @Binds fun bindIrcUserPersister(impl: IrcUserPersister): IrcUserClientApi
  @Binds fun bindNetworkConfigPersister(impl: NetworkConfigPersister): NetworkConfigClientApi
  @Binds fun bindNetworkPersister(impl: NetworkPersister): NetworkClientApi
  @Binds fun bindRpcPersister(impl: RpcPersister): RpcClientApi
  @Binds fun bindProxy(impl: ProxyImpl): Proxy
}

@Singleton
class QuasselApiClient @Inject constructor(
  val aliasManager: AliasManagerServerApi,
  val backlogManager: BacklogManagerServerApi,
  val bufferSyncer: BufferSyncerServerApi,
  val bufferViewConfig: BufferViewConfigServerApi,
  val bufferViewManager: BufferViewManagerServerApi,
  val certManager: CertManagerServerApi,
  val highlightRuleManager: HighlightRuleManagerServerApi,
  val identity: IdentityServerApi,
  val ignoreListManager: IgnoreListManagerServerApi,
  val ircListHelper: IrcListHelperServerApi,
  val network: NetworkServerApi,
  val networkConfig: NetworkConfigServerApi,
)

@Singleton
@Component(modules = [ClientModule::class, ClientDispatcherModule::class, ClientProxyModule::class])
interface ClientComponent {
  fun sync(): SyncHandler
  fun rpc(): RpcDispatcher
  fun api(): QuasselApiClient
}

class QuasselApiTest {
  @Test
  fun test() {
    val client = DaggerClientComponent.builder().build()
    client.sync().invoke(
      "AliasManager",
      ObjectName(""),
      "update",
      listOf(qVariant(emptyMap<String, QVariant_>(), QtType.QVariantMap))
    )
    assertFailsWith<RpcInvocationFailedException.UnknownMethodException> {
      client.sync().invoke(
        "AliasManager",
        ObjectName(""),
        "unknown",
        listOf(qVariant(emptyMap<String, QVariant_>(), QtType.QVariantMap))
      )
    }
    assertFails {
      client.sync().invoke("AliasManager", ObjectName(""), "update", emptyList())
    }
    client.api().aliasManager.requestUpdate(emptyMap())
    client.api().aliasManager.addAlias("name", "expansion")
    client.api().backlogManager.requestBacklog(BufferId(-1))
    client.api().bufferSyncer.requestSetLastSeenMsg(BufferId(1), MsgId(1337))
    client.api().bufferViewConfig.requestSetBufferViewName(ObjectName("1"), "test")
    client.api().bufferViewManager.requestDeleteBufferView(1)
    client.api().certManager.requestUpdate(ObjectName(""), emptyMap())
    client.api().highlightRuleManager.requestRemoveHighlightRule(5)
    client.api().identity.requestUpdate(ObjectName(""), emptyMap())
    println("hi!")
  }

  @Test
  fun testDb() = runBlocking {
    val db = Room.inMemoryDatabaseBuilder<AppDatabase>()
      .setDriver(BundledSQLiteDriver())
      .setQueryCoroutineContext(Dispatchers.IO)
      .build()

    val dao = db.getDao()
    println(dao.count())
    dao.insert(
      TodoEntity(
        title = "kids",
        content = "bring kids home from school"
      )
    )
    println(dao.count())
  }
}
