/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator.visitors

import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.FileSpec
import com.squareup.kotlinpoet.FunSpec
import com.squareup.kotlinpoet.KModifier
import com.squareup.kotlinpoet.ParameterSpec
import com.squareup.kotlinpoet.PropertySpec
import com.squareup.kotlinpoet.TypeSpec
import com.squareup.kotlinpoet.asTypeName
import com.squareup.kotlinpoet.buildCodeBlock
import de.justjanne.libquassel.generator.Constants
import de.justjanne.libquassel.generator.Constants.TYPENAME_GENERATED
import de.justjanne.libquassel.generator.Constants.TYPENAME_QVARIANTLIST
import de.justjanne.libquassel.generator.Constants.TYPENAME_QVARIANT_INTOORTHROW
import de.justjanne.libquassel.generator.Constants.TYPENAME_RPCDISPATCHER
import de.justjanne.libquassel.generator.Constants.TYPENAME_SYNCDISPATCHER
import de.justjanne.libquassel.generator.Constants.TYPENAME_UNKNOWN_METHOD_EXCEPTION
import de.justjanne.libquassel.generator.kotlinmodel.KotlinModel
import de.justjanne.libquassel.generator.rpcmodel.RpcModel
import de.justjanne.libquassel.generator.rpcmodel.RpcModelVisitor
import de.justjanne.libquassel.generator.util.kotlinpoet.ArgString
import de.justjanne.libquassel.generator.util.kotlinpoet.buildWhen
import de.justjanne.libquassel.generator.util.kotlinpoet.withIndent
import de.justjanne.libquassel.generator.util.transformName

class DispatcherGenerator : RpcModelVisitor<KotlinModel?> {
  override fun visitObjectModel(
    model: RpcModel.ObjectModel,
  ): KotlinModel.FileModel {
    val name = toClassName(model)
    return KotlinModel.FileModel(
      listOf(model.source),
      FileSpec.builder(name.packageName, name.simpleName)
        .addImport(
          TYPENAME_QVARIANT_INTOORTHROW.packageName,
          TYPENAME_QVARIANT_INTOORTHROW.simpleName,
        )
        .addAnnotation(TYPENAME_GENERATED)
        .addType(
          TypeSpec.classBuilder(name.simpleName)
            .primaryConstructor(
              FunSpec.constructorBuilder()
                .addAnnotation(Constants.TYPENAME_JAVAX_INJECT)
                .addParameter(
                  ParameterSpec.builder("api", model.name)
                    .build()
                )
                .build()
            )
            .addProperty(
              PropertySpec.builder("api", model.name)
                .initializer("api")
                .addModifiers(KModifier.PRIVATE)
                .build()
            )
            .addSuperinterface(if (model.rpcName.isEmpty()) TYPENAME_RPCDISPATCHER else TYPENAME_SYNCDISPATCHER)
            .addAnnotation(TYPENAME_GENERATED)
            .addFunction(
              FunSpec.builder("invoke")
                .addModifiers(KModifier.OVERRIDE, KModifier.OPERATOR)
                .addAnnotation(TYPENAME_GENERATED)
                .let {
                  if (model.rpcName.isNotEmpty()) {
                    it.addParameter(
                      ParameterSpec.builder(
                        "objectName",
                        Constants.TYPENAME_OBJECTNAME,
                      ).build(),
                    )
                  } else it
                }.addParameter(
                  ParameterSpec.builder(
                    "method",
                    String::class.asTypeName(),
                  ).build(),
                ).addParameter(
                  ParameterSpec.builder(
                    "params",
                    TYPENAME_QVARIANTLIST,
                  ).build(),
                )
                .addCode(
                  buildCodeBlock {
                    buildWhen("method") {
                      for (method in model.methods) {
                        val block =
                          method.accept(this@DispatcherGenerator)
                            as? KotlinModel.FunctionModel
                            ?: continue
                        addCase(ArgString("%S", method.rpcName ?: method.name), block.data)
                      }
                      buildElse {
                        addStatement("throw %T(%S, method)", TYPENAME_UNKNOWN_METHOD_EXCEPTION, model.rpcName)
                      }
                    }
                  },
                )
                .build(),
            )
            .build(),
        ).build(),
    )
  }

  override fun visitFunctionModel(
    model: RpcModel.FunctionModel,
  ) = KotlinModel.FunctionModel(
    model.source,
    buildCodeBlock {
      if (model.static && model.parameters.isEmpty()) {
        addStatement("api.${model.name}()")
      } else {
        addStatement("api.${model.name}(")
        withIndent {
          if (!model.static) {
            addStatement(
              "objectName,",
            )
          }
          for ((i, parameter) in model.parameters.withIndex()) {
            addStatement(
              "${parameter.name} = params[$i].intoOrThrow<%T>(),",
              parameter.type,
            )
          }
        }
        addStatement(")")
      }
    },
  )

  override fun visitParameterModel(
    model: RpcModel.ParameterModel,
  ): KotlinModel? = null

  companion object {
    fun toClassName(model: RpcModel.ObjectModel) = ClassName(
      TYPENAME_SYNCDISPATCHER.packageName,
      "${model.rpcName}${transformName(model.side.name)}Dispatcher",
    )
  }
}
