/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.generator.visitors

import com.squareup.kotlinpoet.ClassName
import com.squareup.kotlinpoet.FileSpec
import com.squareup.kotlinpoet.FunSpec
import com.squareup.kotlinpoet.KModifier
import com.squareup.kotlinpoet.MemberName
import com.squareup.kotlinpoet.ParameterSpec
import com.squareup.kotlinpoet.PropertySpec
import com.squareup.kotlinpoet.TypeSpec
import com.squareup.kotlinpoet.buildCodeBlock
import com.squareup.kotlinpoet.withIndent
import de.justjanne.libquassel.generator.Constants
import de.justjanne.libquassel.generator.Constants.TYPENAME_GENERATED
import de.justjanne.libquassel.generator.Constants.TYPENAME_PROXY
import de.justjanne.libquassel.generator.annotation.RpcFunctionAnnotation
import de.justjanne.libquassel.generator.kotlinmodel.KotlinModel
import de.justjanne.libquassel.generator.rpcmodel.RpcModel
import de.justjanne.libquassel.generator.rpcmodel.RpcModelVisitor
import de.justjanne.libquassel.generator.util.transformName

class ProxyGenerator : RpcModelVisitor<KotlinModel?> {
  override fun visitObjectModel(
    model: RpcModel.ObjectModel,
  ): KotlinModel.FileModel {
    val name = toClassName(model)

    /*
import de.justjanne.libquassel.protocol.api.ObjectName
import de.justjanne.libquassel.protocol.api.client.AliasManagerClientApi
import de.justjanne.libquassel.protocol.api.proxy.Proxy
import de.justjanne.libquassel.protocol.models.types.QtType
import de.justjanne.libquassel.protocol.variant.qVariant
import javax.inject.Inject

class AliasManagerClientApiProxy @Inject constructor(
  private val proxy: Proxy
): AliasManagerClientApi {
  override fun addAlias(name: String, expansion: String) =
    proxy.sync(
      "AliasManager",
      ObjectName.EMPTY,
      "addAlias",
      qVariant(name, QtType.QString),
      qVariant(expansion, QtType.QString),
    )

  override fun requestUpdate(properties: QVariantMap) =
    proxy.sync(
      "AliasManager",
      ObjectName.EMPTY,
      "requestUpdate",
      qVariant(properties, QtType.QVariantMap)
    )
}
     */

    return KotlinModel.FileModel(
      listOf(model.source),
      FileSpec.builder(name.packageName, name.simpleName)
        .addImport(
          "de.justjanne.libquassel.protocol.variant",
          "qVariant",
        )
        .addAnnotation(TYPENAME_GENERATED)
        .addType(
          TypeSpec.classBuilder(name.simpleName)
            .primaryConstructor(
              FunSpec.constructorBuilder()
                .addAnnotation(Constants.TYPENAME_JAVAX_INJECT)
                .addParameter(
                  ParameterSpec.builder("proxy", TYPENAME_PROXY)
                    .build()
                )
                .build()
            )
            .addProperty(
              PropertySpec.builder("proxy", TYPENAME_PROXY)
                .initializer("proxy")
                .addModifiers(KModifier.PRIVATE)
                .build()
            )
            .addSuperinterface(model.name)
            .addAnnotation(TYPENAME_GENERATED)
            .addFunctions(
              model.methods.map { method ->
                FunSpec.builder(method.name)
                  .addModifiers(KModifier.OVERRIDE)
                  .let {
                    if (!method.static) {
                      it.addParameter(
                        ParameterSpec.builder("objectName", Constants.TYPENAME_OBJECTNAME)
                          .build()
                      )
                    } else it
                  }
                  .addParameters(
                    method.parameters.map { param ->
                      ParameterSpec.builder(param.name, param.type)
                        .build()
                    }
                  )
                  .addCode(
                    buildCodeBlock {
                      if (model.rpcName.isEmpty()) {
                        addStatement("proxy.rpc(")
                      } else {
                        addStatement("proxy.sync(")
                      }
                      withIndent {
                        if (model.rpcName.isNotEmpty()) {
                          addStatement("%S,", model.rpcName)
                          if (!method.static) {
                            addStatement("objectName,")
                          } else {
                            addStatement("%T.%N,", Constants.TYPENAME_OBJECTNAME, Constants.MEMBERNAME_OBJECTNAME_EMPTY)
                          }
                        }
                        addStatement("%S,", method.rpcName)
                        for (parameter in method.parameters) {
                          if (parameter.rpcType != null) {
                            addStatement(
                              "%N(%N, %T.%N),",
                              MemberName("de.justjanne.libquassel.protocol.variant", "qVariant"),
                              parameter.name,
                              parameter.rpcType.enclosingClassName!!,
                              parameter.rpcType,
                            )
                          } else {
                            addStatement(
                              "%N(%N),",
                              MemberName("de.justjanne.libquassel.protocol.variant", "qVariant"),
                              parameter.name,
                            )
                          }
                        }
                      }
                      addStatement(")")
                    }
                  )
                  .build()
              }
            )
            .build(),
        ).build(),
    )
  }

  override fun visitFunctionModel(
    model: RpcModel.FunctionModel,
  ) = null

  override fun visitParameterModel(
    model: RpcModel.ParameterModel,
  ): KotlinModel? = null

  companion object {
    fun toClassName(model: RpcModel.ObjectModel) = ClassName(
      TYPENAME_PROXY.packageName,
      "${model.rpcName}${transformName(model.side.name)}Proxy",
    )
  }
}
