/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.serializers.quassel

import de.justjanne.libquassel.protocol.features.FeatureSet
import de.justjanne.libquassel.protocol.io.ChainedByteBuffer
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.models.network.IdentityDto
import de.justjanne.libquassel.protocol.models.types.QtType
import de.justjanne.libquassel.protocol.models.types.QuasselType
import de.justjanne.libquassel.protocol.serializers.PrimitiveSerializer
import de.justjanne.libquassel.protocol.serializers.qt.QVariantMapSerializer
import de.justjanne.libquassel.protocol.variant.QVariantMap
import de.justjanne.libquassel.protocol.variant.into
import de.justjanne.libquassel.protocol.variant.qVariant
import java.nio.ByteBuffer

object IdentitySerializer : PrimitiveSerializer<IdentityDto> {
  override val javaType: Class<IdentityDto> = IdentityDto::class.java

  override fun serialize(
    buffer: ChainedByteBuffer,
    data: IdentityDto,
    featureSet: FeatureSet,
  ) {
    QVariantMapSerializer.serialize(buffer, toMap(data), featureSet)
  }

  override fun deserialize(
    buffer: ByteBuffer,
    featureSet: FeatureSet,
  ): IdentityDto {
    return fromMap(QVariantMapSerializer.deserialize(buffer, featureSet))
  }

  private fun toMap(data: IdentityDto): QVariantMap =
    mapOf(
      "identityId" to qVariant(data.identityId, QuasselType.IdentityId),
      "identityName" to qVariant(data.identityName, QtType.QString),
      "realName" to qVariant(data.realName, QtType.QString),
      "nicks" to qVariant(data.nicks, QtType.QStringList),
      "awayNick" to qVariant(data.awayNick, QtType.QString),
      "awayNickEnabled" to qVariant(data.awayNickEnabled, QtType.Bool),
      "awayReason" to qVariant(data.awayReason, QtType.QString),
      "awayReasonEnabled" to qVariant(data.awayReasonEnabled, QtType.Bool),
      "autoAwayEnabled" to qVariant(data.autoAwayEnabled, QtType.Bool),
      "autoAwayTime" to qVariant(data.autoAwayTime, QtType.Int),
      "autoAwayReason" to qVariant(data.autoAwayReason, QtType.QString),
      "autoAwayReasonEnabled" to qVariant(data.autoAwayReasonEnabled, QtType.Bool),
      "detachAwayEnabled" to qVariant(data.detachAwayEnabled, QtType.Bool),
      "detachAwayReason" to qVariant(data.detachAwayReason, QtType.QString),
      "detachAwayReasonEnabled" to qVariant(data.detachAwayReasonEnabled, QtType.Bool),
      "ident" to qVariant(data.ident, QtType.QString),
      "kickReason" to qVariant(data.kickReason, QtType.QString),
      "partReason" to qVariant(data.partReason, QtType.QString),
      "quitReason" to qVariant(data.quitReason, QtType.QString),
    )

  private fun fromMap(data: QVariantMap) =
    IdentityDto(
      identityId = data["identityId"].into(IdentityId(-1)),
      identityName = data["identityName"].into("<empty>"),
      realName = data["realName"].into(""),
      nicks = data["nicks"].into(listOf("quassel")),
      awayNick = data["awayNick"].into(""),
      awayNickEnabled = data["awayNickEnabled"].into(false),
      awayReason = data["awayReason"].into("Gone fishing."),
      awayReasonEnabled = data["awayReasonEnabled"].into(true),
      autoAwayEnabled = data["autoAwayEnabled"].into(false),
      autoAwayTime = data["autoAwayTime"].into(10),
      autoAwayReason = data["autoAwayReason"].into("Not here. No really. not here!"),
      autoAwayReasonEnabled = data["autoAwayReasonEnabled"].into(false),
      detachAwayEnabled = data["detachAwayEnabled"].into(false),
      detachAwayReason = data["detachAwayReason"].into("All Quassel clients vanished from the face of the earth..."),
      detachAwayReasonEnabled = data["detachAwayReasonEnabled"].into(false),
      ident = data["ident"].into("quassel"),
      kickReason = data["kickReason"].into("Kindergarten is elsewhere!"),
      partReason = data["partReason"].into("http://quassel-irc.org - Chat comfortably. Anywhere."),
      quitReason = data["quitReason"].into("http://quassel-irc.org - Chat comfortably. Anywhere."),
    )
}
