/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.serializers.quassel

import de.justjanne.libquassel.protocol.features.FeatureSet
import de.justjanne.libquassel.protocol.io.ChainedByteBuffer
import de.justjanne.libquassel.protocol.models.network.ChannelModes
import de.justjanne.libquassel.protocol.models.network.IrcChannelDto
import de.justjanne.libquassel.protocol.models.types.QtType
import de.justjanne.libquassel.protocol.serializers.PrimitiveSerializer
import de.justjanne.libquassel.protocol.serializers.qt.QVariantMapSerializer
import de.justjanne.libquassel.protocol.variant.QVariantMap
import de.justjanne.libquassel.protocol.variant.into
import de.justjanne.libquassel.protocol.variant.qVariant
import java.nio.ByteBuffer

object IrcChannelSerializer : PrimitiveSerializer<IrcChannelDto> {
  override val javaType: Class<IrcChannelDto> = IrcChannelDto::class.java

  override fun serialize(
    buffer: ChainedByteBuffer,
    data: IrcChannelDto,
    featureSet: FeatureSet,
  ) {
    QVariantMapSerializer.serialize(buffer, toMap(data), featureSet)
  }

  override fun deserialize(
    buffer: ByteBuffer,
    featureSet: FeatureSet,
  ): IrcChannelDto {
    return fromMap(QVariantMapSerializer.deserialize(buffer, featureSet))
  }

  private fun toMap(data: IrcChannelDto): QVariantMap =
    mapOf(
      "name" to qVariant(data.name, QtType.QString),
      "topic" to qVariant(data.topic, QtType.QString),
      "password" to qVariant(data.password, QtType.QString),
      "encrypted" to qVariant(data.encrypted, QtType.Bool),
      "ChanModes" to
        qVariant(
          data.channelModes.toVariantMap(),
          QtType.QVariantMap,
        ),
      "UserModes" to
        qVariant(
          data.userModes.mapValues { (_, value) ->
            qVariant(value.joinToString(), QtType.QString)
          },
          QtType.QVariantMap,
        ),
    )

  private fun fromMap(data: QVariantMap) =
    IrcChannelDto(
      name = data["name"].into(""),
      topic = data["topic"].into(""),
      password = data["password"].into(""),
      encrypted = data["encrypted"].into(false),
      channelModes =
        data["ChanModes"].into<QVariantMap>()
          ?.let(ChannelModes.Companion::fromVariantMap)
          ?: ChannelModes(),
      userModes =
        data["UserModes"].into<QVariantMap>(emptyMap())
          .mapValues { (_, value) -> value.into<String>()?.toSet().orEmpty() },
    )
}
