/*
 * libquassel
 * Copyright (c) 2024 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */

package de.justjanne.libquassel.protocol.serializers.quassel

import de.justjanne.libquassel.protocol.features.FeatureSet
import de.justjanne.libquassel.protocol.io.ChainedByteBuffer
import de.justjanne.libquassel.protocol.models.network.IrcUserDto
import de.justjanne.libquassel.protocol.models.types.QtType
import de.justjanne.libquassel.protocol.serializers.PrimitiveSerializer
import de.justjanne.libquassel.protocol.serializers.qt.QVariantMapSerializer
import de.justjanne.libquassel.protocol.variant.QVariantMap
import de.justjanne.libquassel.protocol.variant.into
import de.justjanne.libquassel.protocol.variant.qVariant
import org.threeten.bp.Instant
import org.threeten.bp.OffsetDateTime
import org.threeten.bp.ZoneOffset
import java.nio.ByteBuffer

object IrcUserSerializer : PrimitiveSerializer<IrcUserDto> {
  override val javaType: Class<IrcUserDto> = IrcUserDto::class.java

  override fun serialize(
    buffer: ChainedByteBuffer,
    data: IrcUserDto,
    featureSet: FeatureSet,
  ) {
    QVariantMapSerializer.serialize(buffer, toMap(data), featureSet)
  }

  override fun deserialize(
    buffer: ByteBuffer,
    featureSet: FeatureSet,
  ): IrcUserDto {
    return fromMap(QVariantMapSerializer.deserialize(buffer, featureSet))
  }

  private fun toMap(data: IrcUserDto): QVariantMap =
    mapOf(
      "nick" to qVariant(data.nick, QtType.QString),
      "user" to qVariant(data.user, QtType.QString),
      "host" to qVariant(data.host, QtType.QString),
      "realName" to qVariant(data.realName, QtType.QString),
      "account" to qVariant(data.account, QtType.QString),
      "away" to qVariant(data.away, QtType.Bool),
      "awayMessage" to qVariant(data.awayMessage, QtType.QString),
      "idleTime" to qVariant(data.idleTime.atOffset(ZoneOffset.UTC), QtType.QDateTime),
      "loginTime" to qVariant(data.loginTime.atOffset(ZoneOffset.UTC), QtType.QDateTime),
      "server" to qVariant(data.server, QtType.QString),
      "ircOperator" to qVariant(data.ircOperator, QtType.QString),
      "lastAwayMessage" to qVariant(data.lastAwayMessageTime.epochSecond.toInt(), QtType.Int),
      "lastAwayMessageTime" to qVariant(data.lastAwayMessageTime.atOffset(ZoneOffset.UTC), QtType.QDateTime),
      "whoisServiceReply" to qVariant(data.whoisServiceReply, QtType.QString),
      "suserHost" to qVariant(data.suserHost, QtType.QString),
      "encrypted" to qVariant(data.encrypted, QtType.Bool),
      "channels" to qVariant(data.channels.toList(), QtType.QStringList),
      "userModes" to qVariant(data.userModes.joinToString(), QtType.QString),
    )

  private fun fromMap(data: QVariantMap) =
    IrcUserDto(
      nick = data["nick"].into(""),
      user = data["user"].into(""),
      host = data["host"].into(""),
      realName = data["realName"].into(""),
      account = data["account"].into(""),
      away = data["away"].into(false),
      awayMessage = data["awayMessage"].into(""),
      idleTime = data["idleTime"].into(Instant.EPOCH.atOffset(ZoneOffset.UTC)).toInstant(),
      loginTime = data["loginTime"].into(Instant.EPOCH.atOffset(ZoneOffset.UTC)).toInstant(),
      server = data["server"].into(""),
      ircOperator = data["ircOperator"].into(""),
      lastAwayMessageTime =
        data["lastAwayMessageTime"].into<OffsetDateTime>()?.toInstant()
          ?: data["lastAwayMessage"].into<Int>()?.toLong()?.let(Instant::ofEpochSecond)
          ?: Instant.EPOCH,
      whoisServiceReply = data["whoisServiceReply"].into(""),
      suserHost = data["suserHost"].into(""),
      encrypted = data["encrypted"].into(false),
      channels = data["channels"].into(emptySet()),
      userModes = data["userModes"].into(emptySet()),
    )
}
