/*
 * libquassel
 * Copyright (c) 2021 Janne Mareike Koschinski
 *
 * This Source Code Form is subject to the terms of the Mozilla Public License,
 * v. 2.0. If a copy of the MPL was not distributed with this file, You can
 * obtain one at https://mozilla.org/MPL/2.0/.
 */
package de.justjanne.libquassel.protocol.serializers.handshake

import de.justjanne.bitflags.of
import de.justjanne.libquassel.protocol.models.BufferInfo
import de.justjanne.libquassel.protocol.models.HandshakeMessage
import de.justjanne.libquassel.protocol.models.flags.BufferType
import de.justjanne.libquassel.protocol.models.ids.BufferId
import de.justjanne.libquassel.protocol.models.ids.IdentityId
import de.justjanne.libquassel.protocol.models.ids.NetworkId
import de.justjanne.libquassel.protocol.models.network.IdentityDto
import de.justjanne.libquassel.protocol.testutil.byteBufferOf
import de.justjanne.libquassel.protocol.testutil.handshakeSerializerTest
import org.junit.jupiter.api.Tag
import org.junit.jupiter.api.Test

@Tag("HandshakeSerializerTest")
class SessionInitSerializerTest {
  @Test
  fun testEmptyMap() =
    handshakeSerializerTest(
      HandshakeMessage.SessionInit(
        identities = emptyList(),
        bufferInfos = emptyList(),
        networkIds = emptyList(),
      ),
      byteBufferOf(
        // 4 elements
        0x00u, 0x00u, 0x00u, 0x02u,
        // ByteBuffer
        0x00u, 0x00u, 0x00u, 0x0Cu,
        0x00u,
        // 7 bytes
        0x00u, 0x00u, 0x00u, 0x07u,
        // MsgType
        0x4Du, 0x73u, 0x67u, 0x54u, 0x79u, 0x70u, 0x65u,
        // String
        0x00u, 0x00u, 0x00u, 0x0Au,
        0x00u,
        // 11 2-byte chars
        0x00u, 0x00u, 0x00u, 0x16u,
        0x00u, 0x53u, 0x00u, 0x65u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x69u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x49u,
        0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u, 0x74u,
      ),
      featureSets = emptyList(),
      serializeFeatureSet = null,
    )

  @Test
  fun testSimple() =
    handshakeSerializerTest(
      HandshakeMessage.SessionInit(
        identities = emptyList(),
        bufferInfos = emptyList(),
        networkIds = emptyList(),
      ),
      byteBufferOf(
        0x00u, 0x00u, 0x00u, 0x04u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x4Du, 0x73u, 0x67u,
        0x54u, 0x79u, 0x70u, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x16u, 0x00u, 0x53u, 0x00u,
        0x65u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x69u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u,
        0x69u, 0x00u, 0x74u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x53u, 0x65u, 0x73u, 0x73u,
        0x69u, 0x6Fu, 0x6Eu, 0x53u, 0x74u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u, 0x08u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x03u, 0x00u, 0x00u, 0x00u, 0x16u, 0x00u, 0x42u, 0x00u, 0x75u, 0x00u, 0x66u, 0x00u, 0x66u, 0x00u, 0x65u, 0x00u,
        0x72u, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u, 0x66u, 0x00u, 0x6Fu, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x09u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x4Eu, 0x00u, 0x65u, 0x00u, 0x74u, 0x00u, 0x77u,
        0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x6Bu, 0x00u, 0x49u, 0x00u, 0x64u, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x09u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x49u, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u,
        0x6Eu, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x65u, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u,
        0x09u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u,
      ),
    )

  @Test
  fun testRealistic() =
    handshakeSerializerTest(
      HandshakeMessage.SessionInit(
        identities =
          listOf(
            IdentityDto(
                identityId = IdentityId(1),
            ),
          ),
        bufferInfos =
          listOf(
            BufferInfo(
              networkId = NetworkId(4),
              bufferId = BufferId(1337),
              bufferName = "#quassel",
              type = BufferType.of(BufferType.Channel),
            ),
          ),
        networkIds =
          listOf(
            NetworkId(4),
          ),
      ),
      byteBufferOf(
        0x00u, 0x00u, 0x00u, 0x04u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x07u, 0x4Du, 0x73u, 0x67u,
        0x54u, 0x79u, 0x70u, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x16u, 0x00u, 0x53u, 0x00u,
        0x65u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x69u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u,
        0x69u, 0x00u, 0x74u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Cu, 0x53u, 0x65u, 0x73u, 0x73u,
        0x69u, 0x6Fu, 0x6Eu, 0x53u, 0x74u, 0x61u, 0x74u, 0x65u, 0x00u, 0x00u, 0x00u, 0x08u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x03u, 0x00u, 0x00u, 0x00u, 0x16u, 0x00u, 0x42u, 0x00u, 0x75u, 0x00u, 0x66u, 0x00u, 0x66u, 0x00u, 0x65u, 0x00u,
        0x72u, 0x00u, 0x49u, 0x00u, 0x6Eu, 0x00u, 0x66u, 0x00u, 0x6Fu, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x09u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x7Fu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Bu, 0x42u, 0x75u, 0x66u,
        0x66u, 0x65u, 0x72u, 0x49u, 0x6Eu, 0x66u, 0x6Fu, 0x00u, 0x00u, 0x00u, 0x05u, 0x39u, 0x00u, 0x00u, 0x00u, 0x04u,
        0x00u, 0x02u, 0xFFu, 0xFFu, 0xFFu, 0xFFu, 0x00u, 0x00u, 0x00u, 0x08u, 0x23u, 0x71u, 0x75u, 0x61u, 0x73u, 0x73u,
        0x65u, 0x6Cu, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x4Eu, 0x00u, 0x65u, 0x00u, 0x74u, 0x00u, 0x77u, 0x00u, 0x6Fu,
        0x00u, 0x72u, 0x00u, 0x6Bu, 0x00u, 0x49u, 0x00u, 0x64u, 0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x09u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x7Fu, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x4Eu, 0x65u, 0x74u, 0x77u,
        0x6Fu, 0x72u, 0x6Bu, 0x49u, 0x64u, 0x00u, 0x00u, 0x00u, 0x00u, 0x04u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x49u,
        0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x74u, 0x00u, 0x69u, 0x00u, 0x65u,
        0x00u, 0x73u, 0x00u, 0x00u, 0x00u, 0x09u, 0x00u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x7Fu, 0x00u,
        0x00u, 0x00u, 0x00u, 0x09u, 0x49u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x74u, 0x79u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x13u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u,
        0x69u, 0x00u, 0x74u, 0x00u, 0x79u, 0x00u, 0x49u, 0x00u, 0x64u, 0x00u, 0x00u, 0x00u, 0x7Fu, 0x00u, 0x00u, 0x00u,
        0x00u, 0x0Bu, 0x49u, 0x64u, 0x65u, 0x6Eu, 0x74u, 0x69u, 0x74u, 0x79u, 0x49u, 0x64u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x01u, 0x00u, 0x00u, 0x00u, 0x18u, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u,
        0x69u, 0x00u, 0x74u, 0x00u, 0x79u, 0x00u, 0x4Eu, 0x00u, 0x61u, 0x00u, 0x6Du, 0x00u, 0x65u, 0x00u, 0x00u, 0x00u,
        0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Eu, 0x00u, 0x3Cu, 0x00u, 0x65u, 0x00u, 0x6Du, 0x00u, 0x70u, 0x00u, 0x74u,
        0x00u, 0x79u, 0x00u, 0x3Eu, 0x00u, 0x00u, 0x00u, 0x10u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x6Cu,
        0x00u, 0x4Eu, 0x00u, 0x61u, 0x00u, 0x6Du, 0x00u, 0x65u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u, 0x63u, 0x00u, 0x6Bu, 0x00u, 0x73u, 0x00u,
        0x00u, 0x00u, 0x0Bu, 0x00u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x0Eu, 0x00u, 0x71u, 0x00u, 0x75u,
        0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x00u, 0x00u, 0x10u, 0x00u, 0x61u,
        0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x4Eu, 0x00u, 0x69u, 0x00u, 0x63u, 0x00u, 0x6Bu, 0x00u, 0x00u,
        0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x1Eu, 0x00u, 0x61u, 0x00u, 0x77u, 0x00u,
        0x61u, 0x00u, 0x79u, 0x00u, 0x4Eu, 0x00u, 0x69u, 0x00u, 0x63u, 0x00u, 0x6Bu, 0x00u, 0x45u, 0x00u, 0x6Eu, 0x00u,
        0x61u, 0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x14u, 0x00u, 0x61u, 0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u,
        0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x1Au,
        0x00u, 0x47u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x65u, 0x00u, 0x20u, 0x00u, 0x66u, 0x00u, 0x69u, 0x00u, 0x73u,
        0x00u, 0x68u, 0x00u, 0x69u, 0x00u, 0x6Eu, 0x00u, 0x67u, 0x00u, 0x2Eu, 0x00u, 0x00u, 0x00u, 0x22u, 0x00u, 0x61u,
        0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu,
        0x00u, 0x6Eu, 0x00u, 0x45u, 0x00u, 0x6Eu, 0x00u, 0x61u, 0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x65u, 0x00u, 0x64u,
        0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x1Eu, 0x00u, 0x61u, 0x00u, 0x75u, 0x00u, 0x74u,
        0x00u, 0x6Fu, 0x00u, 0x41u, 0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x45u, 0x00u, 0x6Eu, 0x00u, 0x61u,
        0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u,
        0x00u, 0x18u, 0x00u, 0x61u, 0x00u, 0x75u, 0x00u, 0x74u, 0x00u, 0x6Fu, 0x00u, 0x41u, 0x00u, 0x77u, 0x00u, 0x61u,
        0x00u, 0x79u, 0x00u, 0x54u, 0x00u, 0x69u, 0x00u, 0x6Du, 0x00u, 0x65u, 0x00u, 0x00u, 0x00u, 0x02u, 0x00u, 0x00u,
        0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x1Cu, 0x00u, 0x61u, 0x00u, 0x75u, 0x00u, 0x74u, 0x00u, 0x6Fu, 0x00u,
        0x41u, 0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u,
        0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x3Cu, 0x00u, 0x4Eu, 0x00u, 0x6Fu,
        0x00u, 0x74u, 0x00u, 0x20u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x2Eu, 0x00u, 0x20u,
        0x00u, 0x4Eu, 0x00u, 0x6Fu, 0x00u, 0x20u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x6Cu, 0x00u, 0x6Cu,
        0x00u, 0x79u, 0x00u, 0x2Eu, 0x00u, 0x20u, 0x00u, 0x6Eu, 0x00u, 0x6Fu, 0x00u, 0x74u, 0x00u, 0x20u, 0x00u, 0x68u,
        0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x21u, 0x00u, 0x00u, 0x00u, 0x2Au, 0x00u, 0x61u, 0x00u, 0x75u,
        0x00u, 0x74u, 0x00u, 0x6Fu, 0x00u, 0x41u, 0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x52u, 0x00u, 0x65u,
        0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x45u, 0x00u, 0x6Eu, 0x00u, 0x61u, 0x00u, 0x62u,
        0x00u, 0x6Cu, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x22u,
        0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x74u, 0x00u, 0x61u, 0x00u, 0x63u, 0x00u, 0x68u, 0x00u, 0x41u, 0x00u, 0x77u,
        0x00u, 0x61u, 0x00u, 0x79u, 0x00u, 0x45u, 0x00u, 0x6Eu, 0x00u, 0x61u, 0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x65u,
        0x00u, 0x64u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u, 0x00u, 0x00u, 0x00u, 0x00u, 0x20u, 0x00u, 0x64u, 0x00u, 0x65u,
        0x00u, 0x74u, 0x00u, 0x61u, 0x00u, 0x63u, 0x00u, 0x68u, 0x00u, 0x41u, 0x00u, 0x77u, 0x00u, 0x61u, 0x00u, 0x79u,
        0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x0Au,
        0x00u, 0x00u, 0x00u, 0x00u, 0x74u, 0x00u, 0x41u, 0x00u, 0x6Cu, 0x00u, 0x6Cu, 0x00u, 0x20u, 0x00u, 0x51u, 0x00u,
        0x75u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x20u, 0x00u, 0x63u, 0x00u,
        0x6Cu, 0x00u, 0x69u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u, 0x73u, 0x00u, 0x20u, 0x00u, 0x76u, 0x00u,
        0x61u, 0x00u, 0x6Eu, 0x00u, 0x69u, 0x00u, 0x73u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x20u, 0x00u,
        0x66u, 0x00u, 0x72u, 0x00u, 0x6Fu, 0x00u, 0x6Du, 0x00u, 0x20u, 0x00u, 0x74u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u,
        0x20u, 0x00u, 0x66u, 0x00u, 0x61u, 0x00u, 0x63u, 0x00u, 0x65u, 0x00u, 0x20u, 0x00u, 0x6Fu, 0x00u, 0x66u, 0x00u,
        0x20u, 0x00u, 0x74u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x20u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x72u, 0x00u,
        0x74u, 0x00u, 0x68u, 0x00u, 0x2Eu, 0x00u, 0x2Eu, 0x00u, 0x2Eu, 0x00u, 0x00u, 0x00u, 0x2Eu, 0x00u, 0x64u, 0x00u,
        0x65u, 0x00u, 0x74u, 0x00u, 0x61u, 0x00u, 0x63u, 0x00u, 0x68u, 0x00u, 0x41u, 0x00u, 0x77u, 0x00u, 0x61u, 0x00u,
        0x79u, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x45u, 0x00u,
        0x6Eu, 0x00u, 0x61u, 0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x65u, 0x00u, 0x64u, 0x00u, 0x00u, 0x00u, 0x01u, 0x00u,
        0x00u, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x69u, 0x00u, 0x64u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u, 0x74u, 0x00u,
        0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x0Eu, 0x00u, 0x71u, 0x00u, 0x75u, 0x00u, 0x61u, 0x00u, 0x73u,
        0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x6Bu, 0x00u, 0x69u, 0x00u, 0x63u,
        0x00u, 0x6Bu, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u,
        0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x34u, 0x00u, 0x4Bu, 0x00u, 0x69u, 0x00u, 0x6Eu, 0x00u, 0x64u, 0x00u,
        0x65u, 0x00u, 0x72u, 0x00u, 0x67u, 0x00u, 0x61u, 0x00u, 0x72u, 0x00u, 0x74u, 0x00u, 0x65u, 0x00u, 0x6Eu, 0x00u,
        0x20u, 0x00u, 0x69u, 0x00u, 0x73u, 0x00u, 0x20u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u,
        0x77u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x21u, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u,
        0x70u, 0x00u, 0x61u, 0x00u, 0x72u, 0x00u, 0x74u, 0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u,
        0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x0Au, 0x00u, 0x00u, 0x00u, 0x00u, 0x68u, 0x00u, 0x68u, 0x00u, 0x74u,
        0x00u, 0x74u, 0x00u, 0x70u, 0x00u, 0x3Au, 0x00u, 0x2Fu, 0x00u, 0x2Fu, 0x00u, 0x71u, 0x00u, 0x75u, 0x00u, 0x61u,
        0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u, 0x6Cu, 0x00u, 0x2Du, 0x00u, 0x69u, 0x00u, 0x72u, 0x00u, 0x63u,
        0x00u, 0x2Eu, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x67u, 0x00u, 0x20u, 0x00u, 0x2Du, 0x00u, 0x20u, 0x00u, 0x43u,
        0x00u, 0x68u, 0x00u, 0x61u, 0x00u, 0x74u, 0x00u, 0x20u, 0x00u, 0x63u, 0x00u, 0x6Fu, 0x00u, 0x6Du, 0x00u, 0x66u,
        0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x74u, 0x00u, 0x61u, 0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x79u, 0x00u, 0x2Eu,
        0x00u, 0x20u, 0x00u, 0x41u, 0x00u, 0x6Eu, 0x00u, 0x79u, 0x00u, 0x77u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x72u,
        0x00u, 0x65u, 0x00u, 0x2Eu, 0x00u, 0x00u, 0x00u, 0x14u, 0x00u, 0x71u, 0x00u, 0x75u, 0x00u, 0x69u, 0x00u, 0x74u,
        0x00u, 0x52u, 0x00u, 0x65u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x6Fu, 0x00u, 0x6Eu, 0x00u, 0x00u, 0x00u, 0x0Au,
        0x00u, 0x00u, 0x00u, 0x00u, 0x68u, 0x00u, 0x68u, 0x00u, 0x74u, 0x00u, 0x74u, 0x00u, 0x70u, 0x00u, 0x3Au, 0x00u,
        0x2Fu, 0x00u, 0x2Fu, 0x00u, 0x71u, 0x00u, 0x75u, 0x00u, 0x61u, 0x00u, 0x73u, 0x00u, 0x73u, 0x00u, 0x65u, 0x00u,
        0x6Cu, 0x00u, 0x2Du, 0x00u, 0x69u, 0x00u, 0x72u, 0x00u, 0x63u, 0x00u, 0x2Eu, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u,
        0x67u, 0x00u, 0x20u, 0x00u, 0x2Du, 0x00u, 0x20u, 0x00u, 0x43u, 0x00u, 0x68u, 0x00u, 0x61u, 0x00u, 0x74u, 0x00u,
        0x20u, 0x00u, 0x63u, 0x00u, 0x6Fu, 0x00u, 0x6Du, 0x00u, 0x66u, 0x00u, 0x6Fu, 0x00u, 0x72u, 0x00u, 0x74u, 0x00u,
        0x61u, 0x00u, 0x62u, 0x00u, 0x6Cu, 0x00u, 0x79u, 0x00u, 0x2Eu, 0x00u, 0x20u, 0x00u, 0x41u, 0x00u, 0x6Eu, 0x00u,
        0x79u, 0x00u, 0x77u, 0x00u, 0x68u, 0x00u, 0x65u, 0x00u, 0x72u, 0x00u, 0x65u, 0x00u, 0x2Eu,
      ),
    )
}
